#|
Copyright (c) 2006 Risto Laakso
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|#
(in-package :nio-httpd)

(defparameter +dyn-pre+ (string-to-octets "<html><body>"))
(defparameter +dyn-post+ (string-to-octets "<br/><br/><img src=\"logo.png\"><h5>Page created by <a href=\"http://common-lisp.net/project/nio\">NIO-HTTPD</a>.</h5></body></html>"))


(defvar +pages-served+ 0)

(defun dynamic-gen (method url data hdrs-start client)
  (declare (ignore method url data hdrs-start))
  (destructuring-bind (family host port) (remote-info client)
  (serve-content client :ok "text/html"
		 (concatenate 'vector
			      +dyn-pre+
	       
			      (string-to-octets
			       (with-output-to-string (str)
				 (princ "<h2>Dynamic test page</h2>" str)
				 
				 (princ "Today is " str)
				 (multiple-value-bind (sec min hrs day mon yrs) (decode-universal-time (get-universal-time))
				   (declare (ignore sec min hrs))
				   (loop for elt in (list day "." mon "." yrs) do (princ elt str)))

				 (princ "<br/>" str)
				 
				 (princ "I have now served " str)
				 (princ (incf +pages-served+) str)
				 (princ " pages.<br/>" str)

				 (princ "Your request is IPv" str)
				 (princ (if (equal family :inet6) "6" "4") str)
				 (princ ", from ip " str)
				 (princ host str)
				 (princ " port " str)
				 (princ port str)
				 (princ ".<br/><br/>" str)

				 str))

			      +dyn-post+))))

(defun main-page (method url data hdrs-start client)
  (declare (ignore method url data hdrs-start))

  (serve-content client :ok "text/html"
		 (string-to-octets
		  (concatenate 'string
			       "<html><body>"
			       "<h2>Welcome!</h2>"
			       "<p>This is NIO-HTTPD 04-02-2006, a non-blocking i/o http-server written in Common Lisp.</p>"
			       "<p>The project homepage is at <a href=\"http://common-lisp.net/project/nio/\">common-lisp.net</a>.</p>"
			       "<br/><br/>"
			       "<img src=\"logo.png\">"
			       "</body></html>"
			       ))))

(defun png-logo (method url data hdrs-start client)
  (declare (ignore method url data hdrs-start))
  (serve-content client :ok "image/png" (file-to-octets "logo120x80.png")))


;; register pages
(setf (gethash (string-to-octets "/") +page-hash+) #'main-page
      (gethash (string-to-octets "/dyn") +page-hash+) #'dynamic-gen
      (gethash (string-to-octets "/logo.png") +page-hash+) #'png-logo)